unit OptionsDialog;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,SynEditTypes,SynEditMiscClasses,
  contnrs, ExtCtrls, Buttons, ComCtrls, StdCtrls, ColorBox, SynEditHighlighter,SynGutterLineNumber,typinfo,
  SynMemo, SynHighlighterXML, SynEdit,SynEditMarkupHighAll,SynEditMarkupWordGroup,EditorOptions;

type
  TElement = class
    Name:String;
    Data:TObject;
  end; // TElement

  { TElements }
  TElements = class
  private
    fName: String;
    function GetElement(aIndex: Integer): TElement;

  protected

  public
    fItems:TObjectList;
    constructor Create(const aName:String);
    destructor Destroy; override;
    procedure AddItem(const aName:String; aData:TObject);
    procedure AddToTreeView(const aTreeView:TTreeView);
    property Items[aIndex:Integer]:TElement read GetElement; default;
    property Name:String read fName write fName;
  published

  end; // TElements

  { TOptionsDialogFrm }
  TOptionsDialogFrm = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    CheckBox5: TCheckBox;
    CheckBox6: TCheckBox;
    ColorBox1: TColorBox;
    ColorBox2: TColorBox;
    ColorBox3: TColorBox;
    ComboBox1: TComboBox;
    ComboBox2: TComboBox;
    ComboBox3: TComboBox;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Notebook1: TNotebook;
    Page1: TPage;
    Panel1: TPanel;
    Panel2: TPanel;
    Splitter1: TSplitter;
    SynEdit1: TSynEdit;
    TreeView1: TTreeView;
    TreeView2: TTreeView;
    procedure CheckBox4Click(Sender: TObject);
    procedure ColorBox2Select(Sender: TObject);
    procedure ComboBox1Select(Sender: TObject);
    procedure ComboBox2Select(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SynEdit1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TreeView2SelectionChanged(Sender: TObject);
  private
    { private declarations }
  public
    ElementsText:TElements;
    ElementsGatter:TElements;
    function GetAdditionalAttributeName(aha:TAdditionalHilightAttribute): string;
    procedure GetDefaultsForPascalAttribute(Attr: TSynHighlightElement; const SynColorScheme: String);
    function GetColorScheme(const SynColorScheme: String): TPascalColorScheme;
    { public declarations }
  end; 

var
  OptionsDialogFrm: TOptionsDialogFrm;

implementation

uses Unit1;

{ TOptionsDialogFrm }
procedure TOptionsDialogFrm.ComboBox1Select(Sender: TObject);
var
  i:integer;
  h:TPLHighlighter;
  p:TTreeNode;
  str:String;
begin
  SynEdit1.lines.clear;
  h:=Form1.HighlighterS[ComboBox1.ItemIndex];
  TreeView2.SortType:=stText;
  TreeView2.Items.BeginUpdate;
  TreeView2.items.Clear;
  p:=TreeView2.Items.Add(nil,h.Highlighter.LanguageName);
  for i:=0 to h.Highlighter.AttrCount-1 do begin
    TreeView2.Items.AddChildObject(p,h.Highlighter.Attribute[i].Name,h.HighlighterTemp.Attribute[i]);
  end;
  ElementsText.AddToTreeView(TreeView2);
  ElementsGatter.AddToTreeView(TreeView2);
  TreeView2.Items.EndUpdate;
  TreeView2.FullExpand;
  if h.SampleSourceCode = '' then
    SynEdit1.Text:=h.HighlighterTemp.SampleSource
  else
    SynEdit1.Text:=h.SampleSourceCode;
  SynEdit1.Highlighter:=h.HighlighterTemp;
  ComboBox2.ItemIndex:=0;
  ComboBox2Select(Sender);
  str:=h.HighlighterTemp.DefaultFilter;
  ComboBox3.Text:=Copy(str,Pos('|',str)+1,Length(str));
end;

procedure TOptionsDialogFrm.ComboBox2Select(Sender: TObject);
  function CheckAttri(const aName:String):Boolean;
  begin
    result:=False;
    if aName = 'Assembler' then result:=True;
    if aName = 'Comment' then result:=True;
    if aName = 'Directive' then result:=True;
    if aName = 'Reserved word' then result:=True;
    if aName = 'Number' then result:=True;
    if aName = 'String' then result:=True;
    if aName = 'Symbol' then result:=True;
  end; // CheckAttri

  function getAttriName(const aName:String):TPascalHilightAttribute;
  begin
    if aName = 'Assembler' then result:=phaAssembler;
    if aName = 'Comment' then result:=phaComment;
    if aName = 'Directive' then result:=phaDirective;
    if aName = 'Reserved word' then result:=phaReservedWord;
    if aName = 'Number' then result:=phaNumber;
    if aName = 'String' then result:=phaString;
    if aName = 'Symbol' then result:=phaSymbol;
  end; // getAttriName
var
  AttriName: String;
  Scheme: TPascalColorScheme;
  ai: TPascalHilightAttribute;
//  ai: TAdditionalHilightAttribute;
  h:TPLHighlighter;
  i:IntegeR;
  Attr:TSynHighlightElement;
begin
  if ComboBox2.ItemIndex >= 0 then begin
    Scheme:=ColorSchemeFactory.GetColorScheme(ComboBox2.Text);
    h:=Form1.HighlighterS[ComboBox1.ItemIndex];
    for i:=0 to h.HighlighterTemp.AttrCount-1 do begin
      Attr:=h.HighlighterTemp.Attribute[i];
      AttriName:=attr.StoredName;
      ai:=getAttriName(AttriName);
      if CheckAttri(AttriName) then begin
        if Scheme.Attributes[ai].FG = clDefault then
          Attr.Foreground := Scheme.Default.FG
        else
          Attr.Foreground := Scheme.Attributes[ai].FG;

        if Scheme.Attributes[ai].BG = clDefault then
          Attr.Background := Scheme.Default.BG
        else
          Attr.Background := Scheme.Attributes[ai].BG;

        if Scheme.Attributes[ai].FC = clDefault then
          Attr.FrameColor := Scheme.Default.FC
        else
          Attr.FrameColor := Scheme.Attributes[ai].FC;
        Attr.Style := Scheme.Attributes[ai].Styles;
        Attr.StyleMask := Scheme.Attributes[ai].StylesMask;
      end
      else begin
        Attr.Background:=Scheme.Default.BG;
        Attr.Foreground:=Scheme.Default.FG;
        Attr.FrameColor:=Scheme.Default.FC;
      end;
    end; // for i
  end;
  SynEdit1.Highlighter:=h.HighlighterTemp;
end;

procedure TOptionsDialogFrm.ColorBox2Select(Sender: TObject);
var
  a:TSynHighlighterAttributes;
  s:TSynSelectedColor;
begin
  if TreeView2.Items.Count > 0 then begin
    if Assigned(TreeView2.Selected) then begin
      if TObject(TreeView2.Selected.Data) is TSynHighlighterAttributes then begin
        a:=TSynHighlighterAttributes(TreeView2.Selected.Data);
        a.Foreground:=ColorBox1.Selected; // Vordergrundfarbe
        a.Background:=ColorBox2.Selected; // Hintergrundfarbe
        a.FrameColor:=ColorBox3.Selected; // Rahmenfarbe
      end
      else begin
        if (TSynSelectedColor(TreeView2.Selected.Data) is TSynSelectedColor) then begin
          s:=TSynSelectedColor(TreeView2.Selected.Data);
          s.Foreground:=ColorBox1.Selected; // Vordergrundfarbe
          s.Background:=ColorBox2.Selected; // Hintergrundfarbe
          s.FrameColor:=ColorBox3.Selected; // Rahmenfarbe
        end
      end;
    end
    else begin
  //    SynEdit1.Gutter.Color:=ColorBox1.Selected;
    end;
  end;
end;

procedure TOptionsDialogFrm.CheckBox4Click(Sender: TObject);
var
  f:TFontStyles;
  a:TSynHighlighterAttributes;
  s:TSynSelectedColor;
begin
  if CheckBox6.Checked then include(f,fsBold);
  if CheckBox5.Checked then include(f,fsItalic);
  if CheckBox4.Checked then include(f,fsUnderline);

  if TreeView2.Items.Count > 0 then begin
    if Assigned(TreeView2.Selected) then begin
      if TObject(TreeView2.Selected.Data) is TSynHighlighterAttributes then begin
        a:=TSynHighlighterAttributes(TreeView2.Selected.Data);
        a.Style:=f
//        a.Foreground:=ColorBox1.Selected; // Vordergrundfarbe
//        a.Background:=ColorBox2.Selected; // Hintergrundfarbe
//        a.FrameColor:=ColorBox3.Selected; // Rahmenfarbe
      end
      else begin
        if (TSynSelectedColor(TreeView2.Selected.Data) is TSynSelectedColor) then begin
          s:=TSynSelectedColor(TreeView2.Selected.Data);
          s.Style:=f;
//          s.Foreground:=ColorBox1.Selected; // Vordergrundfarbe
  //        s.Background:=ColorBox2.Selected; // Hintergrundfarbe
    //      s.FrameColor:=ColorBox3.Selected; // Rahmenfarbe
        end
      end;
    end
    else begin
  //    SynEdit1.Gutter.Color:=ColorBox1.Selected;
    end;
  end;
end;

procedure TOptionsDialogFrm.FormCreate(Sender: TObject);
var
  m:TSynSelectedColor;
begin
  ColorSchemeFactory.GetRegisteredSchemes(ComboBox2.Items);
  ElementsText:=TElements.Create('Text');
    ElementsText.AddItem('TextBlock',SynEdit1.Gutter.Parts[0].MarkupInfo);
    ElementsText.AddItem('IncrementalSearch',SynEdit1.IncrementColor);
    ElementsText.AddItem('HighlightAll',SynEdit1.HighlightAllColor);
    ElementsText.AddItem('BracketMatch',SynEdit1.BracketMatchColor);
    ElementsText.AddItem('MouseLink',SynEdit1.MouseLinkColor);
    ElementsText.AddItem('HighlightWord',SynEdit1.MarkupByClass[TSynEditMarkupHighlightAllCaret].MarkupInfo);
    ElementsText.AddItem('WordGroup',SynEdit1.MarkupByClass[TSynEditMarkupWordGroup].MarkupInfo);
    ElementsText.AddItem('LineHighlight',SynEdit1.LineHighlightColor);

  ElementsGatter:=TElements.Create('Gatter');
    ElementsGatter.AddItem('LineNumber',SynEdit1.Gutter.Parts[1].MarkupInfo);
    ElementsGatter.AddItem('ModifiedLine',SynEdit1.Gutter.Parts[2].MarkupInfo);
    ElementsGatter.AddItem('FoldingTree',SynEdit1.Gutter.Parts[4].MarkupInfo);
    ElementsGatter.AddItem('FoldedCode',SynEdit1.FoldedCodeColor);
    ElementsGatter.AddItem('GutterSeparator',SynEdit1.Gutter.Parts[3].MarkupInfo);
//    ElementsGatter.AddItem('Gutter',nil);
end;

procedure TOptionsDialogFrm.SynEdit1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Attri: TSynHighlighterAttributes;
  Token: String;
  MouseXY, XY: TPoint;
begin
  MouseXY := Point(X - (SynEdit1.CharWidth div 2), Y);
  XY := SynEdit1.PixelsToRowColumn(MouseXY);
  Token := ''; Attri := nil;
  SynEdit1.GetHighlighterAttriAtRowCol(XY, Token, Attri);
  if Assigned(Attri) then
    TreeView2.Selected:=TreeView2.items[0].Items[TreeView2.Items[0].IndexOfText(Attri.Name)]
  else
    TreeView2.Selected:=TreeView2.items[0].Items[TreeView2.Items[0].IndexOfText('Space')];
end;

procedure TOptionsDialogFrm.TreeView2SelectionChanged(Sender: TObject);
var
  a:TSynHighlighterAttributes;
  s:TSynSelectedColor;
begin
  if TreeView2.Items.Count > 0 then begin
    if Assigned(TreeView2.Selected) then begin
      if TObject(TreeView2.Selected.Data) is TSynHighlighterAttributes then begin
        a:=TSynHighlighterAttributes(TreeView2.Selected.Data);
        ColorBox1.Selected:=a.Foreground ; // Vordergrundfarbe
        ColorBox2.Selected:=a.Background ; // Hintergrundfarbe
        ColorBox3.Selected:=a.FrameColor ; // Rahmenfarbe
        CheckBox6.Checked:=fsBold in a.Style;
        CheckBox5.Checked:=fsItalic in a.Style;
        CheckBox4.Checked:=fsUnderline in a.Style;
      end
      else begin
        if (TSynSelectedColor(TreeView2.Selected.Data) is TSynSelectedColor) then begin
          s:=TSynSelectedColor(TreeView2.Selected.Data);
          ColorBox1.Selected:=s.Foreground ; // Vordergrundfarbe
          ColorBox2.Selected:=s.Background ; // Hintergrundfarbe
          ColorBox3.Selected:=s.FrameColor ; // Rahmenfarbe
        end;
      end;
    end
    else begin
  //    ColorBox1.Selected:=SynEdit1.Gutter.Color;
    end;
  end;
end;

function TOptionsDialogFrm.GetAdditionalAttributeName(
  aha: TAdditionalHilightAttribute): string;
begin
  result:=GetEnumName(TypeInfo(TAdditionalHilightAttribute), ord(aha));
end; // TOptionsDialogFrm.GetAdditionalAttributeName

procedure TOptionsDialogFrm.GetDefaultsForPascalAttribute(
  Attr: TSynHighlightElement; const SynColorScheme: String);
var
  AttriName: String;
  Scheme: TPascalColorScheme;
  pha: TPascalHilightAttribute;
  aha: TAdditionalHilightAttribute;
begin
  AttriName := Attr.StoredName;
  if AttriName = '' then
    exit;

  Scheme := GetColorScheme(SynColorScheme);

  for pha := low(pha) to High(pha) do begin
    if AttriName <> PascalHilightAttributeNames[pha] then Continue;
    if Scheme.Attributes[pha].FG = clDefault
    then Attr.Foreground := Scheme.Default.FG
    else Attr.Foreground := Scheme.Attributes[pha].FG;
    if Scheme.Attributes[pha].BG = clDefault
    then Attr.Background := Scheme.Default.BG
    else Attr.Background := Scheme.Attributes[pha].BG;
    if Scheme.Attributes[pha].FC = clDefault
    then Attr.FrameColor := Scheme.Default.FC
    else Attr.FrameColor := Scheme.Attributes[pha].FC;
    //DebugLn(['TEditorOptions.GetDefaultsForPascalAttribute SynColorScheme=',SynColorScheme,' AttriName=',AttriName,' BG=',ColorToString(Scheme.Attributes[pha].BG),' Background=',ColorToString(Attr.Background),' SchemeBG=',ColorToString(Scheme.Default.BG)]);
    Attr.Style := Scheme.Attributes[pha].Styles;
    Attr.StyleMask := Scheme.Attributes[pha].StylesMask;
    Exit;
  end;

  for aha := low(aha) to High(aha) do
  begin
    if AttriName <> GetAdditionalAttributeName(aha) then Continue;
    if Scheme.Additional[aha].FG = clDefault
    then Attr.Foreground := Scheme.Default.FG
    else Attr.Foreground := Scheme.Additional[aha].FG;
    if Scheme.Additional[aha].BG = clDefault
    then Attr.Background := Scheme.Default.BG
    else Attr.Background := Scheme.Additional[aha].BG;
    if Scheme.Additional[aha].FC = clDefault
    then Attr.FrameColor := Scheme.Default.FC
    else Attr.FrameColor := Scheme.Additional[aha].FC;
    Attr.Style := Scheme.Additional[aha].Styles;
    Attr.StyleMask := Scheme.Additional[aha].StylesMask;
    Exit;
  end;

  Attr.Foreground := Scheme.Default.FG;
  Attr.Background := Scheme.Default.BG;
  Attr.FrameColor := Scheme.Default.FC;
  Attr.Style := Scheme.Default.Styles;
  Attr.StyleMask := Scheme.Default.StylesMask;
end; // TOptionsDialogFrm.GetDefaultsForPascalAttribute

function TOptionsDialogFrm.GetColorScheme(const SynColorScheme: String
  ): TPascalColorScheme;
begin
  Result := ColorSchemeFactory.GetColorScheme(SynColorScheme);
end; // TOptionsDialogFrm.GetColorScheme

function TElements.GetElement(aIndex: Integer): TElement;
begin
  result:=fItems[aIndex] as TElement;
end; // TElements.GetElement

{ TElements }
constructor TElements.Create(const aName:String);
begin
  inherited Create;
  fItems:=TObjectList.Create;
  Name:=aName;
end; // TElements.Create

destructor TElements.Destroy;
begin
  inherited Destroy;
end; // TElements.Destroy

procedure TElements.AddItem(const aName: String; aData:TObject);
var
  item:TElement;
begin
  item:=TElement.Create;
  item.Name:=aName;
  item.Data:=aData;
  fItems.Add(item);
end; // TElements.AddItem

procedure TElements.AddToTreeView(const aTreeView: TTreeView);
var
  i:Integer;
  p:TTreeNode;
begin
  p:=aTreeView.items.Add(nil,Name);
  for i:=0 to fItems.Count-1 do begin
    aTreeView.items.AddChildObject(p,items[i].Name,items[i].Data);
  end; // for i
end; // TElements.AddToTreeView

initialization
  {$I OptionsDialog.lrs}

end.

